@@ -9,7 +9,7 @@ use std::ptr::NonNull;
 ///A b-tree with owned nodes
 ///and what makes it different from the BTreeMap in std
 ///is it choose arbitrary number as the maximum number of keys
-///in each node, as long as the number grater or equal to 3
+///in each node, as long as the number greater or equal to 3
 ///Note if the number has been set to 3 it becomes a 2-3 tree
 pub struct BTree<K: Ord, V> {
     root_node: OpNode<K, V>,
@@ -302,7 +302,11 @@ impl<K: Ord, V> Node<K, V> {
     /// The caller must guarantee the `new_d` is not None
     /// this method will put new Data in a sorted place
     /// And if the key exists it will update the value
-    fn adding_data(node: OpNode<K, V>, new_d: Option<Data<K, V>>) -> OpNode<K, V> {
+    fn adding_data(
+        node: OpNode<K, V>,
+        new_d: Option<Data<K, V>>,
+        tree: &mut BTree<K, V>,
+    ) -> OpNode<K, V> {
         let new_d = new_d.unwrap();
         let data = Node::get_inner_data(node);
         let data_size = Node::get_data_size(node);
@@ -328,6 +332,10 @@ impl<K: Ord, V> Node<K, V> {
                         let ordering = x.key.cmp(&new_d.key);
                         match ordering {
                             Ordering::Equal => {
+                                // Given each time the Btree takes in a key-value
+                                // it's length will plus 1 but it's not ok
+                                // so we minus 1 here
+                                tree.len -= 1;
                                 (*inner_d.as_ptr())[idx].value = new_d.value;
                                 break node;
                             }
@@ -387,7 +395,7 @@ impl<K: Ord, V> Node<K, V> {
     /// when the number of keys equals to the maximum
     /// split the node at index of [m / 2] - 1
     /// Note [] here is ceil operation
-    fn split_node(node: OpNode<K, V>, split_idx: usize) -> OpNode<K, V> {
+    fn split_node(node: OpNode<K, V>, split_idx: usize, tree: &mut BTree<K, V>) -> OpNode<K, V> {
         let data = Node::get_inner_data(node);
         let data_size = Node::get_data_size(node);
         let children_size = Node::get_children_size(node);
@@ -398,13 +406,13 @@ impl<K: Ord, V> Node<K, V> {
         let mut right_node: OpNode<K, V> = None;
         while l_n < split_idx {
             let left_data = Node::pop_front_inner_data(data);
-            left_node = Node::adding_data(left_node, left_data);
+            left_node = Node::adding_data(left_node, left_data, tree);
             l_n += 1;
         }
 
         while r_n > 0 {
             let right_data = Node::pop_back_inner_data(data);
-            right_node = Node::adding_data(right_node, right_data);
+            right_node = Node::adding_data(right_node, right_data, tree);
             r_n -= 1;
         }
 
@@ -578,16 +586,26 @@ impl<K: Ord, V> BTree<K, V> {
         loop {
             if cur_node.is_none() {
                 self.len += 1;
-                self.root_node = Node::adding_data(cur_node, Some(Data { key: k, value: v }));
+                self.root_node = Node::adding_data(cur_node, Some(Data { key: k, value: v }), self);
                 break;
             }
             let children_size = Node::get_children_size(cur_node);
             if children_size != 0 {
-                cur_node = Node::moving_target(cur_node, &k);
-                continue;
+                let cur_inner = Node::get_inner_data(cur_node);
+                let contains_k = cur_inner
+                    .as_ref()
+                    .map(|v| unsafe { (*v.as_ptr()).iter().any(|d| d.key.eq(&k)) });
+                if contains_k == Some(true) {
+                    self.len += 1; // because adding_data will minus 1 when equal happens
+                    Node::adding_data(cur_node, Some(Data { key: k, value: v }), self);
+                    break;
+                } else {
+                    cur_node = Node::moving_target(cur_node, &k);
+                    continue;
+                }
             } else {
                 self.len += 1;
-                let added_node = Node::adding_data(cur_node, Some(Data { key: k, value: v }));
+                let added_node = Node::adding_data(cur_node, Some(Data { key: k, value: v }), self);
                 self._up_merging(added_node);
                 break;
             }
@@ -601,7 +619,7 @@ impl<K: Ord, V> BTree<K, V> {
             let parent = Node::get_parent(cur_node);
             let parent_data_size = Node::get_data_size(parent);
             if data_size >= self.max_key_num {
-                let splitted_node = Node::split_node(cur_node, self.min_key_num);
+                let splitted_node = Node::split_node(cur_node, self.min_key_num, self);
                 if parent.is_none() {
                     self.root_node = splitted_node;
                     break;
@@ -1374,7 +1392,11 @@ impl<K: Ord, V> BTree<K, V> {
     /// assert!(b.contains(&2));
     /// ```   
     pub fn contains(&self, k: &K) -> bool {
-        self.iter().any(|n| n.0.eq(k))
+        if self.is_empty() {
+            false
+        } else {
+            self.iter().any(|n| n.0.eq(k))
+        }
     }
 
     /// Removing by key
@@ -1412,18 +1434,33 @@ impl<K: Ord, V> BTree<K, V> {
     pub fn iter<'a>(&'a self) -> Iter<'a, K, V> {
         let seen = HashSet::new();
         let seen_back = HashSet::new();
-        Iter {
-            next_nodes: vec![NextNodes {
-                node: self.root_node,
-                index: Cell::new(0),
-            }],
-            seen: seen,
-            next_back_nodes: vec![NextNodes {
-                node: self.root_node,
-                index: Cell::new(Node::get_data_size(self.root_node) - 1),
-            }],
-            seen_back: seen_back,
-            _marker: PhantomData,
+        let next_back_nodes_start = if self.root_node.is_some() {
+            Node::get_data_size(self.root_node) - 1
+        } else {
+            0
+        };
+        if self.len > 0 {
+            Iter {
+                next_nodes: vec![NextNodes {
+                    node: self.root_node,
+                    index: Cell::new(0),
+                }],
+                seen: seen,
+                next_back_nodes: vec![NextNodes {
+                    node: self.root_node,
+                    index: Cell::new(next_back_nodes_start),
+                }],
+                seen_back: seen_back,
+                _marker: PhantomData,
+            }
+        } else {
+            Iter {
+                next_nodes: Vec::new(),
+                seen: seen,
+                next_back_nodes: Vec::new(),
+                seen_back: seen_back,
+                _marker: PhantomData,
+            }
         }
     }
 
