@@ -58,8 +58,19 @@ mod tests {
         let mut btr = BTree::from_iter(data);
         btr.set(8, 91);
         btr.set(10, 100);
+        assert_eq!(btr.len(), 3);
         let v: Vec<_> = btr.into_iter().collect();
         assert_eq!(v, vec![(8, 91), (10, 100), (12, 12)]);
+
+        let mut btr = BTree::new(3);
+        for (k, v) in [(1, 1), (2, 2), (3, 3)] {
+            btr.insert(k, v);
+        }
+        assert_eq!(btr.get(&2), Some(&2));
+        assert_eq!(btr.len(), 3);
+        btr.insert(2, 20);
+        assert_eq!(btr.get(&2), Some(&20));
+        assert_eq!(btr.len(), 3);
     }
 
     #[test]
@@ -148,4 +159,34 @@ mod tests {
         let v: Vec<_> = btr.into_iter().collect();
         assert_eq!(v, data2);
     }
+
+    #[test]
+    fn btree_empty_iter() {
+        let b: BTree<u32, u32> = BTree::new(3);
+        assert_eq!(b.iter().next(), None);
+        assert_eq!(b.iter().next_back(), None);
+    }
+
+    #[test]
+    fn avl_empty_iter() {
+        let a: AVL<i32, i32> = AVL::new();
+        assert_eq!(a.iter().next(), None);
+        assert_eq!(a.iter().next_back(), None);
+    }
+
+    #[test]
+    fn btree_empty_into_iter() {
+        let b: BTree<u32, u32> = BTree::new(3);
+        let mut iter = b.into_iter();
+        assert_eq!(iter.next(), None);
+        assert_eq!(iter.next_back(), None);
+    }
+
+    #[test]
+    fn avl_empty_into_iter() {
+        let a: AVL<i32, i32> = AVL::new();
+        let mut iter = a.into_iter();
+        assert_eq!(iter.next(), None);
+        assert_eq!(iter.next_back(), None);
+    }
 }
